----------------------------------------------------------------------------------
-- Company: UAM 
-- Design Name: counter
-- Project Name: stopwatch
-- Description: 
--   Parameterizable counter. Being the value of the generic COUNT_LENGTH,
--   the counter counts from 0 to (COUNT_LENGTH-1).
--   The size of the output count port is derived automatically from
--   the generic.
-- Dependencies: 
--   stopwatch_pkg
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.stopwatch_pkg.all;   -- use project's package

-------------------------------------------------------------------------------
-- Entity

entity counter is
  generic (
           COUNT_LENGTH : integer := 100  -- Count 0 to (COUNT_LENGTH-1)
          );
  port (
        -- Inputs:
        Clk       : in std_logic;  -- Clock (rising edge)
        Reset     : in std_logic;  -- Async reset, active high
        SyncReset : in std_logic;  -- Sync reset, active high
        EnableIn  : in std_logic;  -- Clock enable. If 0 count is stopped
        -- Outputs:
        Count     : out std_logic_vector (log2_ceil(COUNT_LENGTH)-1 downto 0);
                                   -- Output count value
        EnableOut : out std_logic  -- '1' when Count is last value and EnableIn
                                   --   (combinational output)
       );
end counter;

-------------------------------------------------------------------------------
-- Architecture

architecture Behavioral of counter is

  -- Internal signal to implement the count:
  signal iCount     : std_logic_vector (log2_ceil(COUNT_LENGTH) - 1 downto 0);
  -- Gets '1' with max count value:          
  signal maxReached : std_logic;
  
begin

  -- Set maxReached when final count value is reached:



  -- Implement the counter (iCount). Reset with maxReached:
 
 
 
 
  -- Assign outputs: Count and EnableOut (active when EnableIn and maxReached):
 
 
 
end Behavioral;

