----------------------------------------------------------------------------------
-- Company: UAM 
-- Design Name: debounce
-- Project Name: stopwatch
-- Description: 
--   Provides anti-bouncing filtering to an input data signal. The filtering is
--   based on three successive samples at a rate defined by EnSample.
-- Dependencies: 
--   (none)
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-------------------------------------------------------------------------------
-- Entity

entity debounce is
    Port ( 
           Clk      : in  std_logic;  -- Clock (rising edge)
           Reset    : in  std_logic;  -- Async reset, active high
           EnSample : in  std_logic;  -- Sample DataIn when high
           DataIn   : in  std_logic;  -- Data input
           DataOut  : out std_logic   -- Filtered data output
          );
end debounce;

-------------------------------------------------------------------------------
-- Architecture

architecture Behavioral of debounce is

  -- Shift register for sampled data. Three stable samples plus one
  --   extra FF for anti-metastability:
  signal sampled : std_logic_vector (3 downto 0);

begin
  
  process (Clk, Reset)
  begin
    if Reset = '1' then
      sampled <= (others => '0');
    elsif rising_edge (Clk) then
      if EnSample = '1' then
        sampled <= sampled (2 downto 0) & DataIn;    
      end if;
    end if;
  end process;

  DataOut <= '1' when sampled (3 downto 1) = "111" else '0';
  
end Behavioral;

